@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
 /* NetworkManager Wireless Applet -- Display wireless access points and allow user control
  *
  * Dan Williams <dcbw@redhat.com>
@@ -24,43 +25,28 @@
 
 void
 helper_fill_secret_entry (NMConnection *connection,
-                          GtkEntry *entry,
+                          GtkBuilder *builder,
+                          const char *entry_name,
                           GType setting_type,
-                          HelperSecretFunc func,
-                          const char *setting_name,
-                          const char *secret_name)
+                          HelperSecretFunc func)
 {
+	GtkWidget *widget;
 	NMSetting *setting;
-	GHashTable *secrets;
-	GError *error = NULL;
-	GValue *value;
 	const char *tmp;
 
 	g_return_if_fail (connection != NULL);
-	g_return_if_fail (entry != NULL);
+	g_return_if_fail (builder != NULL);
+	g_return_if_fail (entry_name != NULL);
 	g_return_if_fail (func != NULL);
-	g_return_if_fail (setting_name != NULL);
-	g_return_if_fail (secret_name != NULL);
 
 	setting = nm_connection_get_setting (connection, setting_type);
-
-	if (nm_connection_get_scope (connection) == NM_CONNECTION_SCOPE_SYSTEM) {
-		if (setting) {
-			tmp = (*func) (setting);
-			if (tmp)
-				gtk_entry_set_text (entry, tmp);
+	if (setting) {
+		tmp = (*func) (setting);
+		if (tmp) {
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, entry_name));
+			g_assert (widget);
+			gtk_entry_set_text (GTK_ENTRY (widget), tmp);
 		}
-	} else {
-		secrets = nm_gconf_get_keyring_items (connection, setting_name, TRUE, &error);
-		if (!secrets) {
-			g_clear_error (&error);
-			return;
-		}
-
-		value = g_hash_table_lookup (secrets, secret_name);
-		if (value)
-			gtk_entry_set_text (entry, g_value_get_string (value));
-		g_hash_table_destroy (secrets);
 	}
 }
 
